%RSCS_Ver2.m is the reach scale channel simulator master script that runs
%the model published by Eaton and Hassan [2013] in JGR-ES. It is based on a
%piece-by-piece accounting structure where all processes are determined
%based on a probability of occurrence combined with a random number
%generator to run Monte-Carlo simulations

clear all; clc;

%SIMULATION PARAMTERS
Nyears = 500;                        %specify the number of years for which to run one simulation
MC_N = 500;                          %specify the number of monte carlo simulations to run

SaveFlag = 1;                       %turn the autosave flag on (1) or off (0)


Qm = [3 5 10 15 20 25 30 35 40 45 50 55 60 65 70 ];

for ScaleIndx = 1:numel(Qm);                      %determines which scale to run the model for, based on the matrix of Qm values
    Qbf = Qm(ScaleIndx);                %specify the bankfull discharge (m3/s)

    %CALCULATE THE REGIME CHANNEL DIMENSIONS AND TRANSPORT RATE BASED ON
    %ANDREWS 1984 CHANNEL
                         
    S = 0.02.*Qbf.^-0.35;              %specify the mean reach gradient (m/m) based on exponent of Andrews 1984 dataset, matched to FTC slope
    D50 = 45;                           %the median surface grain size (mm) used for sediment transport estimates (from average of Andrews 1984 data)
    D84 = 128;                          %the 84th percentile of the surface (mm) used for stability analysis (from average of Andrews 1984 data)
    H = 0.4;                            %the effective rooting depth for the riparian forest (m), paramterizing bank strength
    Reg_Calc = UBCRMH_core(S, Qbf, D50, D84, H, 1);
    
    %CALCULATE BASED ONN EMMETT DATASET, Group 1
%         S = 0.054.*Qbf.^-0.66;
%         D50 = 30;                           %the median surface grain size (mm) used for sediment transport estimates (from average of Andrews 1984 data)
%         D84 = 75;
%         H = 0.4;
%         Reg_Calc = UBCRMH_core(S, Qbf, D50, D84, H, 1);
  
    %CONSTANTS RELATED TO THE STREAM CHANNEL
    W_ch = Reg_Calc(1);                 %this is the long-term average bankfull width
    L_ch = 15.*W_ch;                    %this is the reach length
    d_reg = Reg_Calc(2);                %the mean depth 
    U_reg = Reg_Calc(3);
    Q_sed = Reg_Calc(4);

    H_crit = W_ch./5;                   %assume that any LWD piece less than 1/5th the width leaves the system
    Dur_flood = 24 * 60 * 60;           %the duration of the annual transport event, in seconds
    HLcrit = 80;                       %the critical H to L ratio at which a piece becomes a key memeber as a percentage

    %PRE-DEFINE THE DATA STORAGE MATRICES
    LW_vol = zeros(Nyears, MC_N);
    LW_sp = zeros(Nyears, MC_N);
    LW_hg = zeros(Nyears, MC_N);
    LW_in = zeros(Nyears, MC_N);
    SED_vol = zeros(Nyears, MC_N);
    TotalSed = zeros(Nyears, MC_N);
    PotSed = zeros(Nyears, MC_N);
    JamSed = zeros(Nyears, MC_N);
    JamLW = zeros(Nyears, MC_N);


    %PRE-DEFINE A PROBE VARIABLE TO QUERY OUTPUTS DURING THE SIMULATION
    Probe_1 = zeros(7000,4);
    ip1 = 1;

    %PRE-DEFINE THE VECTOR TO STORE ALL LWD LENGTHS AT THE END OF EACH
    %SIMULATION
    WoodAll = [];
    DiamAll = [];

    %CONSTANTS RELATED TO THE FOREST
    SPH = 500;                         %this is the density of the trees on the floodplain (in Stems Per Ha)
    H_tr = 30;                          %this is the average tree height (m)
    D_tr = 0.40;                        %this is the average tree diameter (m)
    K_decay = 0.01;                     %this is an annual decay rate applied every year to reduce the piece diameter
    K_break = 0.10;                    %an annual breakage rate for LW pieces with H/D = 100

    %CONVERSIONS OF CONSTANTS
    Dens_tr = SPH ./ 100.^2;        %this estimated the tree density by converting stems/Ha to stems/m2

    %SETUP A WAITBAR FOR THE PROGRAM RUN
    G = waitbar(0,'Please wait...');

    %RUN THE MONTE CARLO ROUTINE
    for MCi = 1:MC_N
        
        MCi
        
        waitbar(MCi/MC_N,G)

        %DEFINE OUR LWD MATRIX
        %LWD containts [Length, Diameter, Age, Orientation)
        LWD = zeros(2000,9);               %assume that the max number of peices is 1000 and we will have 5 parameters
            %LWD stores (in order):
                %1. Length, 
                %2. Diameter, 
                %3. Residence Time, 
                %4. Orientation,
                %5. Stored Volume,
                %6. Jam Number (the number code for the jam the peice is
                %associated with, if it is in or is forming a jam)
                %7. Jam age class (4=young, 1 = old)
                %8. LW function (0 = suspended, 0.5 = one side on bank, 1 =
                %in-channel)
                %9. Potential volume of sediment to be stored behind each piece


        %RUN the simulation of the wood load over time
        for year = 1 : Nyears

            %A. GENERATE a series of new LWD pieces to add the the channel,
            %considering the processes of breakage during the input phase, and
            %add them to the data matrix LWD
            [LWD] = Wood_In( H_tr, D_tr, W_ch, L_ch, Dens_tr, LWD);

            %B. REMOVE all small wood from the data matrix LWD
              [LWD] = Wood_Out(LWD, H_crit);

            %C. IDENTIFY all jams in the system
            [LWD, JamInd, Nkey] = Jam_ID(LWD, W_ch, HLcrit, year);

                %specify a filter to get the real data
                Data = find(LWD(:,1) > 0);              

            %do the following only if there are LW pieces in the channel
            if isempty(Data) == 0

                %D. MOVE the LWD pieces that are in the channel
                [LWD] = MoveWood(LWD, Data, W_ch, L_ch, d_reg, Nkey);

                %E. STORE SEDIMENT in association with each wood piece
                [LWD] = StoreSed(LWD, Data, Q_sed, Dur_flood, L_ch, W_ch, S);

                %record the data calculated thus far
                    %wood load (m3/m2)
                    LW_vol(year, MCi) = sum(LWD(Data,1)...  
                        .* LWD(Data,2).^2) ./(L_ch.*W_ch);
                    %wood load for suspended pieces
                    Data_sp = find(LWD(:,8) == 0.05);
                    LW_sp(year, MCi) = sum(LWD(Data_sp,1)...  
                        .* LWD(Data_sp,2).^2) ./(L_ch.*W_ch);
                    %wood load for hanging pieces
                    Data_hg = find(LWD(:,8) == 0.5);
                    LW_hg(year, MCi) = sum(LWD(Data_hg,1)...  
                        .* LWD(Data_hg,2).^2) ./(L_ch.*W_ch);
                    %wood load for in-channel pieces
                    Data_in = find(LWD(:,8) == 0.95);
                    LW_in(year, MCi) = sum(LWD(Data_in,1)...  
                        .* LWD(Data_in,2).^2) ./(L_ch.*W_ch);

                    %relative sediment storage (m3/m3)
                    SED_vol(year, MCi) = sum(LWD(Data,5)) ./ (L_ch.*W_ch*d_reg);

                %F. DECAY AND BREAK instream LW
                [ LWD, Probe_1, ip1 ] = DecayWood( LWD, year, Data, K_decay, K_break, Probe_1, ip1);


                %record the data for the run
                Data = LWD(:,1) > 0;                %re-define the data filter after all changes have been made
                InJam = LWD(:,6) > 0;               %re-define the in-jam filter after all changes


                    %total volume of sediment in the reach 
                    TotalSed(year, MCi) = sum(LWD(Data,5));

                    %total potential volume of stored sediment
                    PotSed(year, MCi) = sum(LWD(Data,9));

                    %total volume of sediment stored in jams
                    JamSed(year, MCi) = sum(LWD(InJam,5));

                    %volume of wood in jams (m3/m2)
                    JamLW(year, MCi) = sum(LWD(InJam,1)...   
                    .* LWD(InJam,2).^2) ./(L_ch.*W_ch);

                    %number jams in the reach
                    if isempty(JamInd) == 0 
                        JamN(year, MCi) = Nkey;
                    else
                        JamN(year, MCi) = 0;
                    end

            end

            %reset all NaN values found in sediment storage term to 0 
            LWD(isnan(LWD(:,5)),5) = 0;     

            %this will stop the program if an NaN value appears
            if sum(isnan(LWD(:,:))) > 0
                break
            end

        end  %end loop for a single simulation


        IsWood = LWD(:,1)~=0;       %find all of the LWD pieces at the end of the run
        WoodAll = vertcat(WoodAll,LWD(IsWood,1));      %store the values for all simulations in WoodAll
        DiamAll = vertcat(DiamAll, LWD(IsWood,2));

    end  %end loop for a number of monte carlo simulations
    close(G)
     Dlim=find(Probe_1(:,3) > 0,1, 'last');

     JamData = Probe_1(1:Dlim, :);

    if SaveFlag == 1
        FileName = ['Results_Q_',num2str(Qbf),'.mat'];         %GENERATE FILE NAMES FOR FOREST DENSITY SENSITIVITY
        save(FileName, 'LW_vol', 'LW_sp', 'LW_hg', 'LW_in', 'SED_vol', 'TotalSed', 'PotSed', 'JamSed', 'JamLW','WoodAll', 'DiamAll', 'JamData', 'Reg_Calc')
    end

end

break;

    
    
    
    
        
    
    
    
    